@@ -1,6 +1,6 @@
 #
 #  This file is part of the Connection-Set Algebra (CSA).
-#  Copyright (C) 2010,2011,2012 Mikael Djurfeldt
+#  Copyright (C) 2010,2011,2012,2020 Mikael Djurfeldt
 #
 #  CSA is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -338,23 +338,26 @@ class NoParIterator ():
         self.subIterator = False
 
     def iterator (self, low0, high0, low1, high1, state):
-        print(low0, high0, low1, high1)
-        if not self.subIterator:
-            self.subIterator = self.noParIterator (state)
-            self.lastC = next (self.subIterator)
-        c = self.lastC
-        while c[1] < low1:
-            c = next (self.subIterator)
-        while c[1] < high1:
-            j = c[1]
-            while c[1] == j and c[0] < low0:
+        try:
+            print(low0, high0, low1, high1)
+            if not self.subIterator:
+                self.subIterator = self.noParIterator (state)
+                self.lastC = next (self.subIterator)
+            c = self.lastC
+            while c[1] < low1:
                 c = next (self.subIterator)
-            while c[1] == j and c[0] < high0:
-                yield c
-                c = next (self.subIterator)
-            while c[1] == j:
-                c = next (self.subIterator)
-        self.lastC = c
+            while c[1] < high1:
+                j = c[1]
+                while c[1] == j and c[0] < low0:
+                    c = next (self.subIterator)
+                while c[1] == j and c[0] < high0:
+                    yield c
+                    c = next (self.subIterator)
+                while c[1] == j:
+                    c = next (self.subIterator)
+            self.lastC = c
+        except StopIteration:
+            return
 
 
 class BinaryMask (BinaryCSAObject, Mask):
@@ -374,19 +377,22 @@ class MaskIntersection (BinaryMask):
         BinaryMask.__init__ (self, '*', op1, op2, 1)
 
     def iterator (self, low0, high0, low1, high1, state):
-        iter1 = self.op1.iterator (low0, high0, low1, high1, state)
-        iter2 = self.op2.iterator (low0, high0, low1, high1, state)
-        (i1, j1) = next (iter1)
-        (i2, j2) = next (iter2)
-        while True:
-            if (j1, i1) < (j2, i2):
-                (i1, j1) = next (iter1)
-            elif (j2, i2) < (j1, i1):
-                (i2, j2) = next (iter2)
-            else:
-                yield (i1, j1)
-                (i1, j1) = next (iter1)
-                (i2, j2) = next (iter2)
+        try:
+            iter1 = self.op1.iterator (low0, high0, low1, high1, state)
+            iter2 = self.op2.iterator (low0, high0, low1, high1, state)
+            (i1, j1) = next (iter1)
+            (i2, j2) = next (iter2)
+            while True:
+                if (j1, i1) < (j2, i2):
+                    (i1, j1) = next (iter1)
+                elif (j2, i2) < (j1, i1):
+                    (i2, j2) = next (iter2)
+                else:
+                    yield (i1, j1)
+                    (i1, j1) = next (iter1)
+                    (i2, j2) = next (iter2)
+        except StopIteration:
+            return
 
 
 class FiniteMaskIntersection (Finite, MaskIntersection):
@@ -403,40 +409,43 @@ class MaskMultisetSum (BinaryMask):
         BinaryMask.__init__ (self, "+", op1, op2, 0)
 
     def iterator (self, low0, high0, low1, high1, state):
-        iter1 = self.op1.iterator (low0, high0, low1, high1, state)
-        iter2 = self.op2.iterator (low0, high0, low1, high1, state)
         try:
-            (i1, j1) = next (iter1)
-        except StopIteration:
-            (i2, j2) = next (iter2)
-            while True:
-                yield (i2, j2)
-                (i2, j2) = next (iter2)
-        try:
-            (i2, j2) = next (iter2)
-        except StopIteration:
-            while True:
-                yield (i1, j1)
+            iter1 = self.op1.iterator (low0, high0, low1, high1, state)
+            iter2 = self.op2.iterator (low0, high0, low1, high1, state)
+            try:
                 (i1, j1) = next (iter1)
-        while True:
-            i1s = i1
-            j1s = j1
-            while (j1, i1) <= (j2, i2):
-                yield (i1, j1)
-                try:
-                    (i1, j1) = next (iter1)
-                except StopIteration:
-                    while True:
-                        yield (i2, j2)
-                        (i2, j2) = next (iter2)
-            while (j2, i2) <= (j1s, i1s):
-                yield (i2, j2)
-                try:
+            except StopIteration:
+                (i2, j2) = next (iter2)
+                while True:
+                    yield (i2, j2)
                     (i2, j2) = next (iter2)
-                except StopIteration:
-                    while True:
-                        yield (i1, j1)
+            try:
+                (i2, j2) = next (iter2)
+            except StopIteration:
+                while True:
+                    yield (i1, j1)
+                    (i1, j1) = next (iter1)
+            while True:
+                i1s = i1
+                j1s = j1
+                while (j1, i1) <= (j2, i2):
+                    yield (i1, j1)
+                    try:
                         (i1, j1) = next (iter1)
+                    except StopIteration:
+                        while True:
+                            yield (i2, j2)
+                            (i2, j2) = next (iter2)
+                while (j2, i2) <= (j1s, i1s):
+                    yield (i2, j2)
+                    try:
+                        (i2, j2) = next (iter2)
+                    except StopIteration:
+                        while True:
+                            yield (i1, j1)
+                            (i1, j1) = next (iter1)
+        except StopIteration:
+            return
 
 
 class FiniteMaskMultisetSum (Finite, MaskMultisetSum):
@@ -455,21 +464,24 @@ class MaskDifference (BinaryMask):
     def iterator (self, low0, high0, low1, high1, state):
         iter1 = self.op1.iterator (low0, high0, low1, high1, state)
         iter2 = self.op2.iterator (low0, high0, low1, high1, state)
-        (i1, j1) = next (iter1)
-        (i2, j2) = next (iter2)
-        while True:
-            if (j1, i1) < (j2, i2):
-                yield (i1, j1)
-                (i1, j1) = next (iter1)
-                continue
-            elif (i1, j1) == (i2, j2):
-                (i1, j1) = next (iter1)
-            try:
-                (i2, j2) = next (iter2)
-            except StopIteration:
-                while True:
+        try:
+            (i1, j1) = next (iter1)
+            (i2, j2) = next (iter2)
+            while True:
+                if (j1, i1) < (j2, i2):
                     yield (i1, j1)
                     (i1, j1) = next (iter1)
+                    continue
+                elif (i1, j1) == (i2, j2):
+                    (i1, j1) = next (iter1)
+                try:
+                    (i2, j2) = next (iter2)
+                except StopIteration:
+                    while True:
+                        yield (i1, j1)
+                        (i1, j1) = next (iter1)
+        except StopIteration:
+            return
 
 
 def cmpPostOrder (c0, op1):
@@ -518,13 +530,16 @@ class ExplicitMask (FiniteMask):
 
     def boundedIterator (self, low0, high0, low1, high1, state):
         iterator = iter (self.connections)
-        (i, j) = next (iterator)
-        while j < low1:
-            (i, j) = next (iterator)
-        while j < high1:
-            if low0 <= i and i < high0:
-                yield (i, j)
+        try:
             (i, j) = next (iterator)
+            while j < low1:
+                (i, j) = next (iterator)
+            while j < high1:
+                if low0 <= i and i < high0:
+                    yield (i, j)
+                (i, j) = next (iterator)
+        except StopIteration:
+            return
 
 
 class IntervalSetMask (Mask):
@@ -553,32 +568,35 @@ class IntervalSetMask (Mask):
 
     def iterator (self, low0, high0, low1, high1, state):
         iterator1 = self.set1.intervalIterator ()
-        i1 = next (iterator1)
-        while i1[1] < low1:
+        try:
             i1 = next (iterator1)
-        while i1[0] < high1:
-            for j in range (max (i1[0], low1), min (i1[1] + 1, high1)):
-                iterator0 = self.set0.intervalIterator ()
-                try:
-                    i0 = next (iterator0)
-                    while i0[1] < low0:
+            while i1[1] < low1:
+                i1 = next (iterator1)
+            while i1[0] < high1:
+                for j in range (max (i1[0], low1), min (i1[1] + 1, high1)):
+                    iterator0 = self.set0.intervalIterator ()
+                    try:
                         i0 = next (iterator0)
-                    if i0[1] < high0:
-                        for i in range (max (i0[0], low0), i0[1] + 1):
-                            yield (i, j)
-                        i0 = next (iterator0)
-                        while i0[1] < high0:
-                            for i in range (i0[0], i0[1] + 1):
+                        while i0[1] < low0:
+                            i0 = next (iterator0)
+                        if i0[1] < high0:
+                            for i in range (max (i0[0], low0), i0[1] + 1):
                                 yield (i, j)
                             i0 = next (iterator0)
-                        for i in range (i0[0], min (i0[1] + 1, high0)):
-                            yield (i, j)
-                    else:
-                        for i in range (max (i0[0], low0), min (i0[1] + 1, high0)):
-                            yield (i, j)
-                except StopIteration:
-                    pass
-            i1 = next (iterator1)
+                            while i0[1] < high0:
+                                for i in range (i0[0], i0[1] + 1):
+                                    yield (i, j)
+                                i0 = next (iterator0)
+                            for i in range (i0[0], min (i0[1] + 1, high0)):
+                                yield (i, j)
+                        else:
+                            for i in range (max (i0[0], low0), min (i0[1] + 1, high0)):
+                                yield (i, j)
+                    except StopIteration:
+                        pass
+                i1 = next (iterator1)
+        except StopIteration:
+            return
 
     def intersection (self, other):
         if isinstance (other, IntervalSetMask):
@@ -728,43 +746,46 @@ class ISetBoundedMask (FiniteMask):
 
     def boundedIterator (self, low0, high0, low1, high1, state):
         iterator1 = self.set1.intervalIterator ()
-        i1 = next (iterator1)
-        while i1[1] < low1:
+        try:
             i1 = next (iterator1)
-        while i1[0] < high1:
-            i1 = (max (i1[0], low1), min (i1[1], high1 - 1))
-            iterator0 = self.set0.intervalIterator ()
-            try:
-                i0 = next (iterator0)
-                while i0[1] < low0:
+            while i1[1] < low1:
+                i1 = next (iterator1)
+            while i1[0] < high1:
+                i1 = (max (i1[0], low1), min (i1[1], high1 - 1))
+                iterator0 = self.set0.intervalIterator ()
+                try:
                     i0 = next (iterator0)
-                if i0[1] < high0:
-                    for e in self.subMask.iterator (max (i0[0], low0),
-                                                    i0[1] + 1,
-                                                    i1[0], i1[1] + 1,
-                                                    state):
-                        yield e
-                    i0 = next (iterator0)
-                    while i0[1] < high0:
-                        for e in self.subMask.iterator (i0[0], i0[1] + 1,
+                    while i0[1] < low0:
+                        i0 = next (iterator0)
+                    if i0[1] < high0:
+                        for e in self.subMask.iterator (max (i0[0], low0),
+                                                        i0[1] + 1,
                                                         i1[0], i1[1] + 1,
                                                         state):
                             yield e
                         i0 = next (iterator0)
-                        for e in self.subMask.iterator (i0[0],
-                                                        min (i0[1] + 1, high0),
-                                                        i1[0], i1[1] + 1,
-                                                        state):
-                            yield e
-                else:
-                        for e in self.subMask.iterator (max (i0[0], low0),
-                                                        min (i0[1] + 1, high0),
-                                                        i1[0], i1[1] + 1,
-                                                        state):
-                            yield e
-            except StopIteration:
-                pass
-            i1 = next (iterator1)
+                        while i0[1] < high0:
+                            for e in self.subMask.iterator (i0[0], i0[1] + 1,
+                                                            i1[0], i1[1] + 1,
+                                                            state):
+                                yield e
+                            i0 = next (iterator0)
+                            for e in self.subMask.iterator (i0[0],
+                                                            min (i0[1] + 1, high0),
+                                                            i1[0], i1[1] + 1,
+                                                            state):
+                                yield e
+                    else:
+                            for e in self.subMask.iterator (max (i0[0], low0),
+                                                            min (i0[1] + 1, high0),
+                                                            i1[0], i1[1] + 1,
+                                                            state):
+                                yield e
+                except StopIteration:
+                    pass
+                i1 = next (iterator1)
+        except StopIteration:
+            return
 
     def repr (self):
         return '%s*%s' % (IntervalSetMask._sets_to_repr (self.set0, self.set1),
@@ -850,17 +871,20 @@ class CSetIntersection (BinaryCSet):
     def iterator (self, low0, high0, low1, high1, state):
         iter1 = self.op1.iterator (low0, high0, low1, high1, state)
         iter2 = self.op2.iterator (low0, high0, low1, high1, state)
-        (i1, j1, v1) = next (iter1)
-        (i2, j2) = next (iter2)
-        while True:
-            if (j1, i1) < (j2, i2):
-                (i1, j1, v1) = next (iter1)
-            elif (j2, i2) < (j1, i1):
-                (i2, j2) = next (iter2)
-            else:
-                yield (i1, j1, v1)
-                (i1, j1, v1) = next (iter1)
-                (i2, j2) = next (iter2)
+        try:
+            (i1, j1, v1) = next (iter1)
+            (i2, j2) = next (iter2)
+            while True:
+                if (j1, i1) < (j2, i2):
+                    (i1, j1, v1) = next (iter1)
+                elif (j2, i2) < (j1, i1):
+                    (i2, j2) = next (iter2)
+                else:
+                    yield (i1, j1, v1)
+                    (i1, j1, v1) = next (iter1)
+                    (i2, j2) = next (iter2)
+        except StopIteration:
+            return        
 
 
 class CSetMultisetSum (BinaryCSets):
@@ -872,37 +896,40 @@ class CSetMultisetSum (BinaryCSets):
         iter1 = self.op1.iterator (low0, high0, low1, high1, state)
         iter2 = self.op2.iterator (low0, high0, low1, high1, state)
         try:
-            (i1, j1, v1) = next (iter1)
-        except StopIteration:
-            (i2, j2, v2) = next (iter2)
-            while True:
-                yield (i2, j2, v2)
-                (i2, j2, v2) = next (iter2)
-        try:
-            (i2, j2, v2) = next (iter2)
-        except StopIteration:
-            while True:
-                yield (i1, j1, v1)
+            try:
                 (i1, j1, v1) = next (iter1)
-        while True:
-            i1s = i1
-            j1s = j1
-            while (j1, i1) <= (j2, i2):
-                yield (i1, j1, v1)
-                try:
-                    (i1, j1, v1) = next (iter1)
-                except StopIteration:
-                    while True:
-                        yield (i2, j2, v2)
-                        (i2, j2, v2) = next (iter2)
-            while (j2, i2) <= (j1s, i1s):
-                yield (i2, j2, v2)
-                try:
+            except StopIteration:
+                (i2, j2, v2) = next (iter2)
+                while True:
+                    yield (i2, j2, v2)
                     (i2, j2, v2) = next (iter2)
-                except StopIteration:
-                    while True:
-                        yield (i1, j1, v1)
+            try:
+                (i2, j2, v2) = next (iter2)
+            except StopIteration:
+                while True:
+                    yield (i1, j1, v1)
+                    (i1, j1, v1) = next (iter1)
+            while True:
+                i1s = i1
+                j1s = j1
+                while (j1, i1) <= (j2, i2):
+                    yield (i1, j1, v1)
+                    try:
                         (i1, j1, v1) = next (iter1)
+                    except StopIteration:
+                        while True:
+                            yield (i2, j2, v2)
+                            (i2, j2, v2) = next (iter2)
+                while (j2, i2) <= (j1s, i1s):
+                    yield (i2, j2, v2)
+                    try:
+                        (i2, j2, v2) = next (iter2)
+                    except StopIteration:
+                        while True:
+                            yield (i1, j1, v1)
+                            (i1, j1, v1) = next (iter1)
+        except StopIteration:
+            return
 
     def intersection (self, other):
         assert isinstance (other, Mask), 'expected Mask operand'
