@@ -16,7 +16,7 @@
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import _misc
+from . import _misc
 
 def disc (r):
     return _misc.Disc (r)
@@ -30,6 +30,9 @@ def gaussian (sigma, cutoff):
 def block (M, N = None):
     return _misc.Block (M, M if N == None else N)
 
+def repeat (M, N = None):
+    return _misc.Repeat (M, M if N == None else N)
+
 transpose = _misc.Transpose ()
 
 def shift (M, N):
